@@ -1,9 +1,19 @@
-from pyg_base import cell, is_date, ulist, logger, cell_clear, as_list, get_DAG, get_GAD, get_cache, descendants
-from pyg_base import cell_item, Dict
-from functools import partial
+from pyg_base import is_date, ulist, logger, as_list, get_cache, Dict, dictable
+from pyg_encoders import cell_root, root_path, pd_read_parquet, pickle_load, pd_read_csv, dictable_decoded
+from pyg_npy import pd_read_npy
 from pyg_cell._types import _get_db, _get_qq, DBS, QQ
-# import networkx as nx
+from pyg_cell._cell import cell, cell_clear, cell_item, cell_output
+from pyg_cell._dag import get_DAG, get_GAD, descendants
 
+from functools import partial
+# import networkx as nx
+import os
+
+_readers = dict(parquet = pd_read_parquet, 
+                pickle = pickle_load, 
+                npy = pd_read_npy, 
+                csv = pd_read_csv,
+                dictable = dictable_decoded)
 
 
 _deleted = 'deleted'
@@ -279,7 +289,7 @@ class db_cell(cell):
         mode : int , dataetime, optional
             if -1, then does not load and clears the GRAPH
             if 0, then will load from database if found. If not found, will return original document
-            if 1, then will throw an exception if no document is found in the database
+            if 1, then will load, if doc not found, data from files (equivalent to db_cell.load_output)
             if mode is a date, will return the version alive at that date 
             The default is 0.
             
@@ -323,9 +333,10 @@ class db_cell(cell):
                     graph[address] = _load_asof(table = db, kwargs = kwargs, deleted = False, qq = None)
                 except Exception:
                     if mode in (1, True):
-                        raise ValueError('no cells found matching %s'%kwargs)
+                        return self.load_output()
                     else:
-                        return self         
+                        return self
+
         if address in graph:
             saved = graph[address] 
             self_updated = self.get(_updated)
@@ -340,6 +351,26 @@ class db_cell(cell):
             self.update(update)
         return self        
 
+    def load_output(self):
+        """
+        We may save the cell data on file system rather than in the database. 
+        This allows us to load some of the data even if the database itself has been wiped clean.
+        This does not support a full data reload at the moment, but if the cells outputs are simple dataframes, then cell.load_output will reconstruct it. 
+        
+        """
+        root = cell_root(self)
+        if root is None:
+            return self
+        path = root_path(self, root)
+        fmt = path.split('.')[-1].lower()
+        n = len(fmt) + 1
+        for output in cell_output(self):
+            for suffix, reader in _readers.items():
+                filename = path[:-n] + '/%s.%s'%(output, suffix)            
+                if os.path.exists(filename):
+                    self[output] = reader(filename)
+        return self
+
     def push(self):        
         me = self._address
         res = self.go() # run me on my own as I am not part of the push
@@ -586,8 +617,26 @@ def get_cell(table = None, db = None, url = None, server = None, deleted = None,
     >>> assert get_cell('test','test', surname = 'brown').name == 'bob'
     """
     _from_memory = kwargs.pop('_from_memory', True)
-    return _get_cell(table = table, db = db, url = url, server = server, deleted = deleted, _from_memory = _from_memory, doc = doc, **kwargs)
-
+    params = dict(table = table, db = db, url = url, server = server, deleted = deleted)
+    params.update(kwargs)
+    multiples = {k : v for k, v in params.items() if isinstance(v, list)}
+    if len(multiples) == 0:
+        return _get_cell(_from_memory = _from_memory, doc = doc, **params)
+    elif len(multiples) == 1:
+        res = []
+        for key, values in multiples.items():
+            for value in values:
+                params.update({key : value})
+                res.append(_get_cell(_from_memory = _from_memory, doc = doc, **params))
+        return res
+    else:
+        res = dictable(params)
+        docs = []
+        for row in res:
+            params.update(row)
+            docs.append(_get_cell(_from_memory = _from_memory, doc = doc, **params))
+        res[doc or _doc] = docs
+        return res
 
 def get_data(table = None, db = None, url = None, server = None, deleted = None, doc = None, **kwargs):
     """
@@ -623,8 +672,13 @@ def get_data(table = None, db = None, url = None, server = None, deleted = None,
     >>> brown = db_cell(db = people, name = 'bob', surname = 'brown', age = 39).save()
     >>> assert get_data('test','test', surname = 'brown') is None
         
-    """    
-    return cell_item(get_cell(table, db = db, url = url, server = server, deleted = deleted, doc = doc, **kwargs), key = 'data')
+    """  
+    cells = get_cell(table, db = db, url = url, server = server, deleted = deleted, doc = doc, **kwargs)
+    if isinstance(cells, dictable):
+        cells[doc or _doc] = cell_item(cells[doc or _doc], key = 'data')
+        return cells
+    else:
+        return cell_item(cells, key = 'data')
 
 def load_data(table = None, db = None, url = None, server = None, deleted = None, doc = None, **kwargs):
     """
